﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Xml.Linq;
using System.Web.Script.Serialization;
using System.Xml.Serialization;
using System.IO;
using System.Text;
using System.Runtime.Serialization.Json;
using System.Reflection;

namespace MvcServiceLibrary
{
    public static class Helpers
    {
        public static ResponseType GetResponseType(this IHttpRequest request)
        {
            ResponseType result = ResponseType.None;

            // The requested format 
            // is specified via the querystring
            string format = request.QueryString["format"];

            if (String.IsNullOrEmpty(format))
            {
                // If no querystring was specified, 
                // assume the default HTML format
                result = ResponseType.Html;
            }
            else
            {
                if (String.Equals(format, "html", StringComparison.InvariantCultureIgnoreCase))
                {
                    result = ResponseType.Html;
                }
                else if (String.Equals(format, "xml", StringComparison.InvariantCultureIgnoreCase))
                {
                    result = ResponseType.Xml;
                }
                else if (String.Equals(format, "json", StringComparison.InvariantCultureIgnoreCase))
                {
                    result = ResponseType.Json;
                }
            }

            return result;
        }

        // OBSOLETE APPROACH - useful for objects marked as [Serializable]
        //public static string ToJSON(this object obj)
        //{
        //    JavaScriptSerializer serializer = new JavaScriptSerializer();
        //    return serializer.Serialize(obj);
        //}

        public static string ToJSON(this object obj)
        {
            DataContractJsonSerializer serializer = new DataContractJsonSerializer(obj.GetType());
            using (MemoryStream ms = new MemoryStream())
            {
                serializer.WriteObject(ms, obj);
                return Encoding.Default.GetString(ms.ToArray());
            }
        }

        public static string ToXml(this object obj)
        {
            XmlSerializer serializer = new XmlSerializer(obj.GetType());
            StringBuilder sb = new StringBuilder();
            using (StringWriter sw = new StringWriter(sb))
            {
                serializer.Serialize(sw, obj);
            }

            return sb.ToString();
        }

        public static void SetStatusOK(this IHttpResponse response)
        {
            response.StatusCode = 200;
        }

        public static void SetStatusServerError(this IHttpResponse response, string statusDescription)
        {
            response.StatusCode = 500;
            response.StatusDescription = statusDescription;
        }

        public static bool IsSerializable(this object obj)
        {
            // Check for the Serializable type attribute 
            return ((obj.GetType().Attributes & TypeAttributes.Serializable) == TypeAttributes.Serializable);
        }
    }
}
