﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Xml.Linq;
using System.Web.Mvc;

namespace MvcServiceLibrary
{
    public class ServiceControllerFactory : IControllerFactory
    {
        #region IControllerFactory Members

        public IController CreateController(RequestContext context, Type controllerType)
        {
            if (context == null)
            {
                throw new ArgumentNullException("context");
            }
            if (controllerType == null)
            {
                throw new ArgumentNullException("controllerType");
            }
            if (!typeof(IController).IsAssignableFrom(controllerType))
            {
                throw new ArgumentException(String.Format("Unable to find controller: {0}", controllerType), "controllerType");
            }

            try
            {
                IController controller = Activator.CreateInstance(controllerType) as IController;

                Controller mvcController = controller as Controller;
                if (mvcController != null)
                {
                    // Assign the service view factory to this controller
                    mvcController.ViewFactory = new ServiceViewFactory();
                }

                return controller;
            }
            catch (Exception ex)
            {
                throw new InvalidOperationException(String.Format("Error creating controller: {0}", controllerType), ex);
            }
        }

        #endregion
    }
}
