﻿using System;
using System.Data;
using System.Configuration;
using System.Linq;
using System.Web;
using System.Xml.Linq;
using System.Web.Mvc;
using System.Runtime.Serialization;
using System.Collections.Generic;

namespace MvcServiceLibrary
{
    public class ServiceViewFactory : WebFormViewFactory
    {
        protected override IView CreateView(ControllerContext controllerContext, string viewName, string masterName, object viewData)
        {
            ResponseType responseType = controllerContext.HttpContext.Request.GetResponseType();

            if (responseType == ResponseType.None)
            {
                // A format was specified that we don't understand - return an error
                return new ErrorSerializedView() { StatusCode = 400, ErrorMessage = "Unrecognized format specified" };
            }

            // We've checked for "None" (meaning an unknown format)
            // Now, if a serialized format was requested, do some additional validation
            if (responseType != ResponseType.Html)
            {
                // Validate that the viewData exists,
                // is serializable, and
                // isn't of type IDictionary<string,object> - the default
                // format of viewData, which says it's serializable but really isn't.
                if ((viewData != null)
                    && (viewData.IsSerializable())
                    && !typeof(IDictionary<string, object>).IsAssignableFrom(viewData.GetType()))
                {
                    switch (responseType)
                    {
                        // Serialize as XML
                        case ResponseType.Xml:
                            return new XmlSerializedView();

                        // Serialize as JSON
                        case ResponseType.Json:
                            return new JsonSerializedView();
                    }
                }
                else
                {
                    // We are unable to serialize the data:
                    // this is a server-side error
                    return new ErrorSerializedView() { StatusCode = 500, ErrorMessage = "Server Error: Unable to create properly formatted content." };
                }
            }

            return base.CreateView(controllerContext, viewName, masterName, viewData);
        }
    }
}
